import { User } from "@prisma/client"
import type { Server as HTTPServer } from "http"
import type { Socket as NetSocket } from "net"
import type { NextApiResponse } from "next"
import type { Server as IOServer, Server } from "socket.io"
import { Socket } from "socket.io-client"

interface SocketServer extends HTTPServer {
  io?: IOServer
}

interface SocketWithIO extends NetSocket {
  server: SocketServer
}

export interface NextApiResponseWithSocket extends NextApiResponse {
  socket: SocketWithIO
}

export interface ServerToClientEvents {
  // noArg: () => void
  // basicEmit: (a: number, b: string, c: Buffer) => void
  // withAck: (d: string, callback: (e: number) => void) => void
  "update-input": (msg: string) => void
  unauthenticated: () => void
  authenticated: () => void
  test2: (test: string) => void
  test: () => void
}

export interface ClientToServerEvents {
  // hello: () => void
  "input-change": (msg: string) => void
  authenticate: (payload: { token: string }) => void
  test: (payload: any) => void
}

interface InterServerEvents {
  // ping: () => void
}

interface SocketData {
  user: User | null
}

export type cServer = Server<
  ClientToServerEvents,
  ServerToClientEvents,
  InterServerEvents,
  SocketData
>

export type cSocket = Socket<ServerToClientEvents, ClientToServerEvents>
