import { GamePropsSchema } from "@lib/zodSchemas"
import { useSession } from "next-auth/react"
import {
  Dispatch,
  ReactNode,
  SetStateAction,
  createContext,
  useContext,
  useEffect,
  useState,
  useMemo,
} from "react"
import { toast } from "react-toastify"

const initialValue = { payload: null, hash: null }

interface gameContext {
  gameProps: GamePropsSchema
  setGameProps: Dispatch<SetStateAction<GamePropsSchema>>
}

export const gameContext = createContext<gameContext>({
  gameProps: initialValue,
  setGameProps: () => {},
})

export function GameContextProvider({ children }: { children: ReactNode }) {
  const [gameProps, setGameProps] = useState<GamePropsSchema>(initialValue)

  const value = useMemo<gameContext>(
    () => ({ gameProps, setGameProps }),
    [gameProps, setGameProps]
  )

  return <gameContext.Provider value={value}>{children}</gameContext.Provider>
}

function useGameState(initial?: GamePropsSchema) {
  const { gameProps, setGameProps } = useContext(gameContext)
  const { data: session, status } = useSession()

  useEffect(() => {
    if (!initial) return
    setGameProps(initial)
  }, [initial, setGameProps])

  useEffect(() => {
    if (status === "loading") return
    if (!session)
      toast("Nicht angemeldet.", {
        toastId: "user",
        position: "top-center",
      })
    else
      toast(session.user.email, {
        toastId: "user",
        position: "top-center",
        icon: session.user.image ? (
          <img
            style={{ transform: "scale(1.5)", borderRadius: "100%" }}
            src={session.user.image}
            alt="profile picture"
          />
        ) : undefined,
      })
  }, [session, status])

  return {
    gameProps,
    setGameProps,
    session,
    status,
  }
}

export default useGameState
