import { authOptions } from "../auth/[...nextauth]"
import { composeBody, gameIncludes } from "./running"
import sendError from "@backend/sendError"
import sendResponse from "@backend/sendResponse"
import { rejectionErrors } from "@lib/backend/errors"
import getPinFromBody from "@lib/backend/getPinFromBody"
import prisma from "@lib/prisma"
import { GamePropsSchema } from "@lib/zodSchemas"
import type { NextApiRequest, NextApiResponse } from "next"
import { getServerSession } from "next-auth"

export default async function join(
  req: NextApiRequest,
  res: NextApiResponse<GamePropsSchema>
) {
  const session = await getServerSession(req, res, authOptions)
  const pin = await getPinFromBody(req, res)

  if (!session?.user) {
    return sendResponse(req, res, rejectionErrors.unauthorized)
  }

  const { email, id } = session.user

  try {
    const game = await prisma.game.findFirst({
      where: {
        gamePin: {
          pin,
        },
      },
    })
    if (!game) {
      return sendResponse(req, res, {
        message: "Spiel existiert nicht",
        statusCode: 404,
        type: ["infoCyan"],
      })
    }

    const user_Game = await prisma.user_Game.create({
      data: {
        gameId: game.id,
        userId: id,
        index: "player2",
      },
      include: {
        game: gameIncludes,
      },
    })

    const body = composeBody(user_Game.game)

    return sendResponse(req, res, {
      message: `User <${email}> joined game: ${game.id}`,
      body,
      type: ["debug", "infoCyan"],
    })
  } catch (err: any) {
    console.log("HERE".red, err.code, err.meta, err.message)
    throw sendError(req, res, rejectionErrors.gameNotFound)
  }
}
