import {
  NextApiResponseWithSocket,
  cServer,
} from "../../interfaces/NextApiSocket"
import prisma from "@lib/prisma"
import colors from "colors"
import { NextApiRequest } from "next"
import { getSession } from "next-auth/react"
import { Server } from "socket.io"

colors.enable()

const SocketHandler = async (
  req: NextApiRequest,
  res: NextApiResponseWithSocket
) => {
  if (res.socket.server.io) {
    console.log("Socket is already running " + req.url)
  } else {
    console.log("Socket is initializing " + req.url)
    const io: cServer = new Server(res.socket.server, { path: "/api/ws2" })

    res.socket.server.io = io

    // io.use(authenticate)
    io.use(async (socket, next) => {
      try {
        const session = await getSession({
          req: socket.request,
        })
        socket.data.user = await prisma.user.findUnique({
          where: {
            id: session?.user.id,
          },
        })
        next()
      } catch (err) {
        console.log("Unauthorized")
        next(new Error("Unauthorized"))
      }
    })

    io.on("connection", (socket) => {
      console.log(`User connected  <${socket.data.user?.email}>`.green)

      socket.on("authenticate", (msg) => {
        // if (!socket.data.isAuthenticated) socket.emit("unauthenticated")
        // socket.broadcast.emit("update-input", msg)
        // io.emit("update-input", msg)
        console.log(msg)
      })

      socket.on("test", (payload) => {
        console.log("Got test:", payload)
        // ...
      })

      socket.emit("test2", "lol")
    })
  }
  res.end()
}

export default SocketHandler
