import { cSocket } from "../interfaces/NextApiSocket"
import useGameState from "@lib/hooks/useGameState"
import { ChangeEventHandler, useEffect, useState } from "react"
import { io } from "socket.io-client"

let socket: cSocket

const Home = () => {
  const [input, setInput] = useState("")
  const [data, setData] = useState("")

  const { status } = useGameState()

  useEffect(() => {
    if (status === "loading") return
    if (!socket) {
      console.log("Socket init.")
      socket = io({
        path: "/api/ws",
      })
    } else console.log("Socket present.")

    socket.on("connect", () => {
      console.log("connected")
    })

    socket.on("authenticated", () => {
      console.log("Yay!")
    })

    socket.on("unauthenticated", () => {
      console.log("No... pls work")
    })

    socket.on("update-input", (msg) => {
      setData(msg)
    })

    return () => {
      socket.off("connect")
      socket.off("authenticated")
      socket.off("unauthenticated")
      socket.off("update-input")
    }
  }, [status, setData])

  const onChangeHandler: ChangeEventHandler<HTMLInputElement> = (e) => {
    setInput(e.target.value)
    socket?.emit("input-change", e.target.value)
  }

  return (
    <>
      <input
        placeholder="Type something"
        value={input}
        onChange={onChangeHandler}
        onDoubleClick={() => setInput(data)}
      />
      <h1>{data}</h1>
    </>
  )
}

export default Home
